/*-----------------
* Copyright (C) 2001-2003, CompHEP Collaboration
*------------------------------------------------------
* $Id: mk_tab.c,v 1.7 2003/06/25 18:46:42 kryukov Exp $
*
* $Log: mk_tab.c,v $
* Revision 1.7  2003/06/25 18:46:42  kryukov
* *** empty log message ***
*
* Revision 1.6  2003/06/17 11:13:59  sherstnv
* mk_tab now is re-linked by user in $WDIR/usr
* It is necessary for replace U-variables if user develops own ones.
*
* Revision 1.5  2003/05/01 12:19:29  sherstnv
* Pri ne otsutstvii file s imenem zadannom v kom.stroke
* vivoditsya kratkiy help.
*
* Revision 1.4  2003/04/22 09:01:22  kryukov
* Apply indent to improve readability of files
*
* Revision 1.3  2003/04/20 17:55:03  kryukov
* Clean up warnings during make
*
*/
#include <stdio.h>
#include <string.h>
#include "tag_reader.h"
#include "tag_parser.h"
#include "event_reader.h"
#include "4_vector.h"
#include "phys_val.h"
/*#include "paragraphs.h"
#include "model.h"*/
#include "e_tools.h"

#define const
#include "out_ext.h"
#undef const

double pvect[400];

static void
wrongParam (int N)
{
  if (N)
    fprintf (stderr, "Wrong parameter %d\n", N);
  else
    fprintf (stderr, "Wrong number of parameters \n");
  fprintf (stderr, "Parameters:\n"
           " 1- file name,\n"
           " 2- name of variable,\n"
           " 3- minimum limit,\n"
           " 4- maximum limit,\n" " 5- number of bins(<=300).\n");
}


static int
CheckFormat (FILE * f)
{
  int NF;
  char ch;

  rewind (f);
  ch = new_symbol (f);
  if (ch == '#')
    {
      ch = new_symbol (f);
      if (ch == '#')
        {
          NF = 2;
        }
      else if (ch == 'C')
        {
          NF = 1;
        }
      else if (ch == 'P')
        {
          NF = 3;
        }
    }
  else
    {
      fprintf (stderr, "CPYTH format error ...");
      NF = -1;
    }
  rewind (f);

  return NF;
}


static void
ShowHelp (int N)
{
  if (N == 0)
    {
      fprintf (stdout, "Short help:\n"
"  Using: mk_tab {name_of_file_with_events} Var Param_min Param_max Nbins\n\n"    
"The program mk_tab constructs a histogram for variable \"Var\" in the limits\n"
"from \"Param_min\" to \"Param_max\". This histogram will have \"Nbins\" bins\n");
    }
  else
    {
      fprintf (stdout, "Long help:\n"
"Using: mk_tab {name_of_file_with_events} Var Param_min Param_max Nbins\n\n"
"The program mk_tab constructs a histogram for variable \"Var\" in the limits\n"
"from \"Param_min\" to \"Param_max\". This histogram will have \"Nbins\" bins"
"Bla-bla-bla...\n");
    }
}


static int
keep_histo (double *hist, double *dhist, char key, char *plist, double minX,
            double maxX, int nbin)
{
  int i;
  char xname[200];
  char yname[200];
  char xunits[100];

  for (i = 0; i < nin_ + nout_; i++)
    {
      if (i == nin_)
        fprintf (stdout, " ->");
      else if (i)
        fprintf (stdout, ",");
      pinf_ (1, i + 1, xname, NULL);
      fprintf (stdout, " %s", xname);
    }

  if (nin_ == 2)
    {
      sprintf (yname, "Diff. cross section [pb");
    }
  else
    {
      sprintf (yname, "Diff. width [GeV");
    }
  fprintf (stdout, "\n");

  xName (key, plist, xname, xunits);
  fprintf (stdout, "x-axis: \"%s\"  from %f to %f N_bins= %d\n", xname, minX,
           maxX, nbin);
  fprintf (stdout, "%s", yname);
  if (xunits[0])
    fprintf (stdout, "/%s", xunits);
  fprintf (stdout, "]");

  for (i = 0; i < nbin; i++)
    {
      fprintf (stdout, "\n%-12E +/-  %-12E", hist[i], dhist[i]);
    }
  fprintf (stdout, "\n");
  return 1;
}


int
main (int argc, char **argv)
{
  int i, j, k, n;
  int nbin;
  int Nproc;
  int format;
  int npr;
  int in;
  int out;
  int max_format = 0;
  long Npoints;
  int *Nevents;
  int *Npartons;
  int **pnum;
  int NF;

  char key;
  char plist[MAXNP];
  char buff[STRSIZE];
  char buf[STRSIZE];

  double minX, maxX;
  double weight, coef;
  double *CSec;
  double *hist;
  double *dhist;
  double **mass;

  string_comnd com;
  tags *head;
  FILE *f;

  if (strcmp (argv[1], "") == 0 || strcmp (argv[1], "-h") == 0) {
    ShowHelp (0);
    return 1;
  }

  if ((strcmp (argv[1], "-help") == 0) || (strcmp (argv[1], "--help") == 0)) {
    ShowHelp (1);
    return 1;
  };

  f = fopen (argv[1], "r");
  if (!f) {
    fprintf (stderr, "***Error! File not found: %s\n", argv[1]);
    return 2;
  }

/* Check number of input parameters and their sence */
  if (argc != 6)  {
    wrongParam (0);
    return 1;
  }

  if (sscanf (argv[3], "%lf", &minX) != 1) {
    wrongParam (2);
    return 1;
  }

  if (sscanf (argv[4], "%lf", &maxX) != 1 || minX >= maxX) {
    wrongParam (3);
    return 1;
  }

  if (sscanf (argv[5], "%d", &nbin) != 1 || nbin <= 0) {
    wrongParam (4);
    return 1;
  }

  hist  = malloc (nbin * sizeof (double));
  dhist = malloc (nbin * sizeof (double));
  if (NULL == hist || NULL == dhist ) {
    fprintf (stderr, "***Error! a problem with memmory allocation...\n");
    return -1;
  }

  for (i = 0; i < nbin; i++) {
    hist[i] = 0;
    dhist[i] = 0;
  }
  Npoints = 0;

  NF = CheckFormat (f);
  if (2 != NF) {
    fprintf (stderr, "***Wrong format of the data file.\n");
    return -1;
  } else {
/*---  New XML-style fromat ---*/
    head = init_cap (1);
    cup_reader (f, head);
    Nproc = number_proc (head);

    Npartons = malloc (Nproc * sizeof (int));
    Nevents = malloc (Nproc * sizeof (int));
    pnum = malloc (Nproc * sizeof (int *));
    CSec = malloc (Nproc * sizeof (double));
    mass = malloc (Nproc * sizeof (double *));

    for (i = 0; i < Nproc; i++) {
      pnum[i] = malloc (100 * sizeof (int));
      sprintf (com.value, "%i", i + 1);

/*---  Get number of partons, cross section and parton's names from process tag  */
      strcpy (com.name, "ID");
      n = get_tag_with_exactcom (0, head, "process", &com);
      if (n == -1) {
        fprintf (stderr, "***Error! The process tag for the %i-th subprocess has been lost.\n", i + 1);
        return -1;
      }
      Npartons[i] = get_ival (0, "Nparton", head->tag[n]);
      CSec[i] = get_fval (0, "CrosSec", head->tag[n]);

/*---  Get number of events from the n_event tag  */
      strcpy (com.name, "IDprocess");
      n = get_tag_with_exactcom (0, head, "n_event", &com);
      if (n == -1) {
        Nevents[i] = get_ival (0, "N", head->tag[n]);
        fprintf (stderr, "***Error! The n_event tag for the %i-th subprocess has been lost.\n", i + 1);
        return -1;
      }
      Nevents[i] = get_ival (0, "N", head->tag[n]);

/*---  Set format numbers for i-th subprocess  */
      sprintf (com.value, "%i", i + 1);
      format = get_tag_with_exactcom (0, head, "format", &com);
      if (format == -1) {
        fprintf (stderr, "***Error! The format tag for the %i-th subprocess has been lost.\n", i + 1);
        exit (1);
      }

      if (head->tag[format]->tagsize > max_format) {
        max_format = head->tag[format]->tagsize;
      }

      strcpy (com.name, "p1.3");
      n = tag_contain_com (&com, head->tag[format]);
      if (n < 1) {
        fprintf (stderr, "******Error in format tag for the %i-th subprocess.\n", i + 1);
        return -1;
      }
      pnum[i][3] = n - 2;
      strcpy (com.name, "p2.3");

      n = tag_contain_com (&com, head->tag[format]);
      if (n < 1) {
        fprintf (stderr, "***Error in format tag for the %i-th subprocess.\n", i + 1);
        return -1;
      }
      pnum[i][7] = n - 2;

      for (k = 2; k < Npartons[i]; k++) {
        for (j = 0; j < 3; j++) {
          sprintf (com.name, "p%i.%i", k + 1, j + 1);
          n = tag_contain_com (&com, head->tag[format]);
          if (n > 1) {
            pnum[i][4 * k + j + 1] = n - 2;
          } else {
            pnum[i][4 * k + j + 1] = 0;
          }
        }
      }

/*---  Get partons masses and names from partons tags  */
      strcpy (com.name, "IDprocess");
      sprintf (com.value, "%i", i + 1);
      n = -1;
      mass[i] = malloc (Npartons[i] * sizeof (double));
      setzeros();
      for (j = 0; j < Npartons[i]; j++) {
        char * name;
        n = get_tag_with_exactcom (n + 1, head, "parton", &com);
        if (n == -1) {
          fprintf (stderr, " Error! The number of parton tags are not equaled to the number\n"
                           "        of parton in process tag for the %i-th subprocess.\n", i + 1);
          return -1;
        }
        in = get_ival (0, "in", head->tag[n]);
        out = get_ival (0, "out", head->tag[n]);
        name = get_cval (0, "name", head->tag[n]);
	if (in) {
          mass[i][in - 1] = get_fval (0, "mass", head->tag[n]);
        }
        if (out) {
          mass[i][out - 1] = get_fval (0, "mass", head->tag[n]);
        }
	getNames1 (in, out, name);
      }
    }

  if (!checkPhysVal (argv[2], &key, plist)) {
    wrongParam (1);
    return 1;
  }

  {
    char **eventword = malloc (max_format * sizeof (char *));
    int debug = 0;
    int err;

    for (i = 0; i < 4 * nin_; i++) {
      pvect[i] = 0;
    }

    fgets (buf, 1000, f);
    while (fgets (buf, 1000, f))
      {
        sscanf (buf, "%i:%[^\n]", &npr, buff);
        npr--;
        err = event_parser (buff, eventword);
        if (err)
          {
            fprintf (stderr, "Error in event %i\n. Exit!", debug);
            return -1;
          }
        pvect[3] = atof (eventword[pnum[npr][3]]);
        pvect[7] = atof (eventword[pnum[npr][7]]);
        for (i = 2; i < Npartons[npr]; i++)
          for (j = 0; j < 3; j++)
            pvect[4 * i + j + 1] =
              atof (eventword[pnum[npr][4 * i + j + 1]]);
        for (i = 0; i < nin_ + nout_; i++)
          pvect[4 * i] = ENERGY (mass[npr][i], pvect + 4 * i + 1);

        weight = CSec[npr] / Nevents[npr];
        i = nbin * (calcPhysVal (key, plist) - minX) / (maxX - minX);
        if (i >= 0 && i < nbin)
          {
            hist[i] += weight;
            dhist[i] += weight * weight;
            Npoints++;
          }
        debug++;
      }
      free (eventword);
  }
  free (Npartons);
  free (Nevents);
  free (pnum);
  free (CSec);
  free (mass);
  } 

  coef = nbin / (maxX - minX);

  for (i = 0; i < nbin; i++) {
    dhist[i] = coef * sqrt (dhist[i] - hist[i] * hist[i] / Npoints);
    hist[i] *= coef;
  }
  keep_histo (hist, dhist, key, plist, minX, maxX, nbin);
  fclose (f);
  free (hist);
  free (dhist);
  return 0;
}

